# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["S3ReadFilesBinaryRunParams"]


class S3ReadFilesBinaryRunParams(TypedDict, total=False):
    access_key_id: Required[str]

    access_key_secret: Required[str]

    lattice_id: Required[str]

    url: Required[str]

    opt_bucket: str

    opt_enable_http: bool

    opt_endpoint: str

    opt_region: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .identity.identity import (
    IdentityResource,
    AsyncIdentityResource,
    IdentityResourceWithRawResponse,
    AsyncIdentityResourceWithRawResponse,
    IdentityResourceWithStreamingResponse,
    AsyncIdentityResourceWithStreamingResponse,
)

__all__ = ["AdminResource", "AsyncAdminResource"]


class AdminResource(SyncAPIResource):
    @cached_property
    def identity(self) -> IdentityResource:
        return IdentityResource(self._client)

    @cached_property
    def with_raw_response(self) -> AdminResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AdminResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AdminResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AdminResourceWithStreamingResponse(self)


class AsyncAdminResource(AsyncAPIResource):
    @cached_property
    def identity(self) -> AsyncIdentityResource:
        return AsyncIdentityResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAdminResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncAdminResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAdminResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncAdminResourceWithStreamingResponse(self)


class AdminResourceWithRawResponse:
    def __init__(self, admin: AdminResource) -> None:
        self._admin = admin

    @cached_property
    def identity(self) -> IdentityResourceWithRawResponse:
        return IdentityResourceWithRawResponse(self._admin.identity)


class AsyncAdminResourceWithRawResponse:
    def __init__(self, admin: AsyncAdminResource) -> None:
        self._admin = admin

    @cached_property
    def identity(self) -> AsyncIdentityResourceWithRawResponse:
        return AsyncIdentityResourceWithRawResponse(self._admin.identity)


class AdminResourceWithStreamingResponse:
    def __init__(self, admin: AdminResource) -> None:
        self._admin = admin

    @cached_property
    def identity(self) -> IdentityResourceWithStreamingResponse:
        return IdentityResourceWithStreamingResponse(self._admin.identity)


class AsyncAdminResourceWithStreamingResponse:
    def __init__(self, admin: AsyncAdminResource) -> None:
        self._admin = admin

    @cached_property
    def identity(self) -> AsyncIdentityResourceWithStreamingResponse:
        return AsyncIdentityResourceWithStreamingResponse(self._admin.identity)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from quarkupy import Quark, AsyncQuark
from quarkupy._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestClassifiers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_list(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        classifier = client.worker.context.classifiers.list()
        assert classifier.is_closed
        assert classifier.json() == {"foo": "bar"}
        assert cast(Any, classifier.is_closed) is True
        assert isinstance(classifier, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_list_with_all_params(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        classifier = client.worker.context.classifiers.list(
            limit=0,
            offset=0,
            source_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert classifier.is_closed
        assert classifier.json() == {"foo": "bar"}
        assert cast(Any, classifier.is_closed) is True
        assert isinstance(classifier, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_list(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        classifier = client.worker.context.classifiers.with_raw_response.list()

        assert classifier.is_closed is True
        assert classifier.http_request.headers.get("X-Stainless-Lang") == "python"
        assert classifier.json() == {"foo": "bar"}
        assert isinstance(classifier, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_list(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        with client.worker.context.classifiers.with_streaming_response.list() as classifier:
            assert not classifier.is_closed
            assert classifier.http_request.headers.get("X-Stainless-Lang") == "python"

            assert classifier.json() == {"foo": "bar"}
            assert cast(Any, classifier.is_closed) is True
            assert isinstance(classifier, StreamedBinaryAPIResponse)

        assert cast(Any, classifier.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve_text(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/text").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        classifier = client.worker.context.classifiers.retrieve_text(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert classifier.is_closed
        assert classifier.json() == {"foo": "bar"}
        assert cast(Any, classifier.is_closed) is True
        assert isinstance(classifier, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_retrieve_text(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/text").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        classifier = client.worker.context.classifiers.with_raw_response.retrieve_text(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert classifier.is_closed is True
        assert classifier.http_request.headers.get("X-Stainless-Lang") == "python"
        assert classifier.json() == {"foo": "bar"}
        assert isinstance(classifier, BinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_retrieve_text(self, client: Quark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/text").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.worker.context.classifiers.with_streaming_response.retrieve_text(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as classifier:
            assert not classifier.is_closed
            assert classifier.http_request.headers.get("X-Stainless-Lang") == "python"

            assert classifier.json() == {"foo": "bar"}
            assert cast(Any, classifier.is_closed) is True
            assert isinstance(classifier, StreamedBinaryAPIResponse)

        assert cast(Any, classifier.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_retrieve_text(self, client: Quark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `classifier_id` but received ''"):
            client.worker.context.classifiers.with_raw_response.retrieve_text(
                "",
            )


class TestAsyncClassifiers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_list(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        classifier = await async_client.worker.context.classifiers.list()
        assert classifier.is_closed
        assert await classifier.json() == {"foo": "bar"}
        assert cast(Any, classifier.is_closed) is True
        assert isinstance(classifier, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_list_with_all_params(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        classifier = await async_client.worker.context.classifiers.list(
            limit=0,
            offset=0,
            source_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert classifier.is_closed
        assert await classifier.json() == {"foo": "bar"}
        assert cast(Any, classifier.is_closed) is True
        assert isinstance(classifier, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_list(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        classifier = await async_client.worker.context.classifiers.with_raw_response.list()

        assert classifier.is_closed is True
        assert classifier.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await classifier.json() == {"foo": "bar"}
        assert isinstance(classifier, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_list(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        async with async_client.worker.context.classifiers.with_streaming_response.list() as classifier:
            assert not classifier.is_closed
            assert classifier.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await classifier.json() == {"foo": "bar"}
            assert cast(Any, classifier.is_closed) is True
            assert isinstance(classifier, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, classifier.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve_text(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/text").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        classifier = await async_client.worker.context.classifiers.retrieve_text(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert classifier.is_closed
        assert await classifier.json() == {"foo": "bar"}
        assert cast(Any, classifier.is_closed) is True
        assert isinstance(classifier, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_retrieve_text(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/text").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        classifier = await async_client.worker.context.classifiers.with_raw_response.retrieve_text(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert classifier.is_closed is True
        assert classifier.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await classifier.json() == {"foo": "bar"}
        assert isinstance(classifier, AsyncBinaryAPIResponse)

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_retrieve_text(self, async_client: AsyncQuark, respx_mock: MockRouter) -> None:
        respx_mock.get("/worker/context/classifiers/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/text").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.worker.context.classifiers.with_streaming_response.retrieve_text(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as classifier:
            assert not classifier.is_closed
            assert classifier.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await classifier.json() == {"foo": "bar"}
            assert cast(Any, classifier.is_closed) is True
            assert isinstance(classifier, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, classifier.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_retrieve_text(self, async_client: AsyncQuark) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `classifier_id` but received ''"):
            await async_client.worker.context.classifiers.with_raw_response.retrieve_text(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from .docling_extractor import (
    DoclingExtractorResource,
    AsyncDoclingExtractorResource,
    DoclingExtractorResourceWithRawResponse,
    AsyncDoclingExtractorResourceWithRawResponse,
    DoclingExtractorResourceWithStreamingResponse,
    AsyncDoclingExtractorResourceWithStreamingResponse,
)

__all__ = ["ExtractorResource", "AsyncExtractorResource"]


class ExtractorResource(SyncAPIResource):
    @cached_property
    def docling_extractor(self) -> DoclingExtractorResource:
        return DoclingExtractorResource(self._client)

    @cached_property
    def with_raw_response(self) -> ExtractorResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return ExtractorResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExtractorResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return ExtractorResourceWithStreamingResponse(self)


class AsyncExtractorResource(AsyncAPIResource):
    @cached_property
    def docling_extractor(self) -> AsyncDoclingExtractorResource:
        return AsyncDoclingExtractorResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncExtractorResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncExtractorResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExtractorResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncExtractorResourceWithStreamingResponse(self)


class ExtractorResourceWithRawResponse:
    def __init__(self, extractor: ExtractorResource) -> None:
        self._extractor = extractor

    @cached_property
    def docling_extractor(self) -> DoclingExtractorResourceWithRawResponse:
        return DoclingExtractorResourceWithRawResponse(self._extractor.docling_extractor)


class AsyncExtractorResourceWithRawResponse:
    def __init__(self, extractor: AsyncExtractorResource) -> None:
        self._extractor = extractor

    @cached_property
    def docling_extractor(self) -> AsyncDoclingExtractorResourceWithRawResponse:
        return AsyncDoclingExtractorResourceWithRawResponse(self._extractor.docling_extractor)


class ExtractorResourceWithStreamingResponse:
    def __init__(self, extractor: ExtractorResource) -> None:
        self._extractor = extractor

    @cached_property
    def docling_extractor(self) -> DoclingExtractorResourceWithStreamingResponse:
        return DoclingExtractorResourceWithStreamingResponse(self._extractor.docling_extractor)


class AsyncExtractorResourceWithStreamingResponse:
    def __init__(self, extractor: AsyncExtractorResource) -> None:
        self._extractor = extractor

    @cached_property
    def docling_extractor(self) -> AsyncDoclingExtractorResourceWithStreamingResponse:
        return AsyncDoclingExtractorResourceWithStreamingResponse(self._extractor.docling_extractor)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .quark import (
    QuarkResource,
    AsyncQuarkResource,
    QuarkResourceWithRawResponse,
    AsyncQuarkResourceWithRawResponse,
    QuarkResourceWithStreamingResponse,
    AsyncQuarkResourceWithStreamingResponse,
)
from .lattice import (
    LatticeResource,
    AsyncLatticeResource,
    LatticeResourceWithRawResponse,
    AsyncLatticeResourceWithRawResponse,
    LatticeResourceWithStreamingResponse,
    AsyncLatticeResourceWithStreamingResponse,
)
from .registry import (
    RegistryResource,
    AsyncRegistryResource,
    RegistryResourceWithRawResponse,
    AsyncRegistryResourceWithRawResponse,
    RegistryResourceWithStreamingResponse,
    AsyncRegistryResourceWithStreamingResponse,
)

__all__ = [
    "QuarkResource",
    "AsyncQuarkResource",
    "QuarkResourceWithRawResponse",
    "AsyncQuarkResourceWithRawResponse",
    "QuarkResourceWithStreamingResponse",
    "AsyncQuarkResourceWithStreamingResponse",
    "LatticeResource",
    "AsyncLatticeResource",
    "LatticeResourceWithRawResponse",
    "AsyncLatticeResourceWithRawResponse",
    "LatticeResourceWithStreamingResponse",
    "AsyncLatticeResourceWithStreamingResponse",
    "RegistryResource",
    "AsyncRegistryResource",
    "RegistryResourceWithRawResponse",
    "AsyncRegistryResourceWithRawResponse",
    "RegistryResourceWithStreamingResponse",
    "AsyncRegistryResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .extractor import Extractor

__all__ = ["ExtractorListResponse"]

ExtractorListResponse: TypeAlias = List[Extractor]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .classifier import Classifier

__all__ = ["ClassifierListResponse"]

ClassifierListResponse: TypeAlias = List[Classifier]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .openai_embedding_run_params import OpenAIEmbeddingRunParams as OpenAIEmbeddingRunParams
from .openai_completion_base_run_params import OpenAICompletionBaseRunParams as OpenAICompletionBaseRunParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .registry.quark_registry_item import QuarkRegistryItem
from .registry.lattice_registry_item import LatticeRegistryItem

__all__ = ["RegistryListResponse"]


class RegistryListResponse(BaseModel):
    lattices: List[LatticeRegistryItem]
    """A list of all Lattices in the registry."""

    lattices_suffix: str
    """The suffix used for Lattice-related API endpoints."""

    quarks: List[QuarkRegistryItem]
    """A list of all Quarks in the registry."""

    quarks_suffix: str
    """The suffix used for Quark-related API endpoints."""

    root_url: str
    """The base URL of the API."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ......_types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......_base_client import make_request_options
from ......types.admin.identity.identity_role import IdentityRole

__all__ = ["IdentityResource", "AsyncIdentityResource"]


class IdentityResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IdentityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return IdentityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IdentityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return IdentityResourceWithStreamingResponse(self)

    def add(
        self,
        identity_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IdentityRole:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not identity_id:
            raise ValueError(f"Expected a non-empty value for `identity_id` but received {identity_id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._put(
            f"/admin/identity/roles/{id}/members/identity/{identity_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IdentityRole,
        )

    def remove(
        self,
        identity_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IdentityRole:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not identity_id:
            raise ValueError(f"Expected a non-empty value for `identity_id` but received {identity_id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._delete(
            f"/admin/identity/roles/{id}/members/identity/{identity_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IdentityRole,
        )


class AsyncIdentityResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIdentityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncIdentityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIdentityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncIdentityResourceWithStreamingResponse(self)

    async def add(
        self,
        identity_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IdentityRole:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not identity_id:
            raise ValueError(f"Expected a non-empty value for `identity_id` but received {identity_id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._put(
            f"/admin/identity/roles/{id}/members/identity/{identity_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IdentityRole,
        )

    async def remove(
        self,
        identity_id: str,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> IdentityRole:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        if not identity_id:
            raise ValueError(f"Expected a non-empty value for `identity_id` but received {identity_id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._delete(
            f"/admin/identity/roles/{id}/members/identity/{identity_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IdentityRole,
        )


class IdentityResourceWithRawResponse:
    def __init__(self, identity: IdentityResource) -> None:
        self._identity = identity

        self.add = to_raw_response_wrapper(
            identity.add,
        )
        self.remove = to_raw_response_wrapper(
            identity.remove,
        )


class AsyncIdentityResourceWithRawResponse:
    def __init__(self, identity: AsyncIdentityResource) -> None:
        self._identity = identity

        self.add = async_to_raw_response_wrapper(
            identity.add,
        )
        self.remove = async_to_raw_response_wrapper(
            identity.remove,
        )


class IdentityResourceWithStreamingResponse:
    def __init__(self, identity: IdentityResource) -> None:
        self._identity = identity

        self.add = to_streamed_response_wrapper(
            identity.add,
        )
        self.remove = to_streamed_response_wrapper(
            identity.remove,
        )


class AsyncIdentityResourceWithStreamingResponse:
    def __init__(self, identity: AsyncIdentityResource) -> None:
        self._identity = identity

        self.add = async_to_streamed_response_wrapper(
            identity.add,
        )
        self.remove = async_to_streamed_response_wrapper(
            identity.remove,
        )
